#include <stdio.h>
#include <ctype.h>
#include "skeleton.h"

#define BLACK_CHAR '-'
#define CHECK_CHAR '!'
#define WIDTH 60

int save_skeleton_state(const char *name,
	unsigned short ix, unsigned short iy, unsigned short iz,
	board_type board, struct number *nums, short *used, int nlines)
{
	FILE *fp;
	int i, j, l, x, y, r = 1;
	
	fp = fopen(name, "w");
	if (! fp) return 1;
	if (fprintf(fp, "%d %d %d\n", COLS, ROWS, nlines) < 0) goto RET;
	if (fprintf(fp, "%hu %hu %hu\n\n", ix, iy, iz) < 0) goto RET;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (board[y][x] != BLACK) {
				if (fprintf(fp, "%u", board[y][x]) < 0) goto RET;
			}
			else {
				if (fputc(BLACK_CHAR, fp) == EOF) goto RET;
			}
		}
		if (fputc('\n', fp) == EOF) goto RET;
	}
	x = l = 0;
	for (i = 0; i < nlines; i ++) {
		x += nums[i].length;
		if (nums[i].length != l || x >= WIDTH) {
			if (fputc('\n', fp) == EOF) goto RET;
			x = nums[i].length;
		}
		else if (i > 0) {
			if (fputc(' ', fp) == EOF) goto RET;
			x ++;
		}
		if (used[i]) {
			if (fputc(CHECK_CHAR, fp) == EOF) goto RET;
		}
		for (j = 0; j < nums[i].length; j ++) {
			if (fprintf(fp, "%u", nums[i].digit[j]) < 0) goto RET;
		}
		l = nums[i].length;
	}
	if (x > 0 && fputc('\n', fp) == EOF) goto RET;
	r = 0;
	RET:
	fclose(fp);
	return r;
}

int load_skeleton_state(const char *name,
	unsigned short *ix, unsigned short *iy, unsigned short *iz,
	board_type board, struct number *nums, short *used)
{
	FILE *fp;
	int c, i, j, n, x, y, w, h, r = 0;
	
	fp = fopen(name, "r");
	if (! fp) return 1;
	if (fscanf(fp, "%d %d %d", &w, &h, &n) == EOF) goto RET;
	if (w != COLS || h != ROWS || n < 1 || n >= MAXLINES) goto RET;
	if (fscanf(fp, "%hu %hu %hu", ix, iy, iz) == EOF) goto RET;
	x = y = 0;
	while ((c = fgetc(fp)) != EOF) {
		if ('0' <= c && c <= '9') {
			board[y][x] = c - '0';
			x ++;
		}
		else if (c == BLACK_CHAR) {
			board[y][x] = -1;
			x ++;
		}
		if (x == w) {
			x = 0;
			y ++;
			if (y == h) break;
		}
	}
	if (c == EOF) goto RET;
	i = j = 0;
	while ((c = fgetc(fp)) != EOF) {
		if ('1' <= c && c <= '9') {
			if (i == MAXLINES || j == MAXLENGTH) goto RET;
			nums[i].digit[j] = c - '0';
			j ++;
			nums[i].length = j;
		}
		else if (c == CHECK_CHAR) {
			if (i == MAXLINES) goto RET;
			used[i] = 1;
		}
		else if (j > 0) {
			i ++;
			j = 0;
			if (i == n) {
				r = n;
				break;
			}
		}
	}
	RET:
	fclose(fp);
	return r;
}

