#include <stdlib.h>
#include <assert.h>
#include "random.h"

/* Wichmann-Hill */

static int ix = 1, iy = 1, iz = 1;

double rand_next(void)
{
	double r;
	
	ix = (int)((171 * ix) % 30269);
	iy = (int)((172 * iy) % 30307);
	iz = (int)((170 * iz) % 30323);
	r = ix / 30269.0 + iy / 30307.0 + iz / 30323.0;
	while (r >= 1) r --;
	return r;
}

void rand_normalize_seed(unsigned short *x, unsigned short *y, unsigned short *z)
{
	if (*x > 30000) *x = (*x - 1) % 30000 + 1;
	if (*y > 30000) *y = (*y - 1) % 30000 + 1;
	if (*z > 30000) *z = (*z - 1) % 30000 + 1;
	if (*x < 1) *x = 1;
	if (*y < 1) *y = 1;
	if (*z < 1) *z = 1;
}

void rand_get_seed(unsigned short *x, unsigned short *y, unsigned short *z)
{
	*x = ix; *y = iy; *z = iz;
}

void rand_set_seed(unsigned short x, unsigned short y, unsigned short z)
{
	ix = x; iy = y; iz = z;
}

unsigned int rand_range(unsigned int max)
{
	return rand_next() * max;
}

int rand_percent(unsigned int n)
{
	return rand_range(100) < n;
}

void rand_shuffle(unsigned short *a, unsigned int n)
{
	unsigned short b;
	unsigned int i, j;
	
	for (i = n; i > 1; i --, a ++) {
		j = rand_range(i);
		assert(j <= i);
		b = a[0];  a[0] = a[j];  a[j] = b;
	}
}

int count_bits(unsigned short n)
{
	n = (n & 0x5555) + (n >> 1 & 0x5555);
	n = (n & 0x3333) + (n >> 2 & 0x3333);
	n = (n & 0x0f0f) + (n >> 4 & 0x0f0f);
	return (n & 0x00ff) + (n >> 8 & 0x00ff);
}

unsigned short rand_select_bit(unsigned short n)
{
	int i, j;
	i = count_bits(n);
	j = rand_range(i);
	for (i = 0; i < j; i ++, n ^= n & - n)
		;
	return n & - n;
}
