#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <time.h>

#include "skeleton.h"
#include "random.h"

void clear_board(board_type b)
{
	int x, y;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			b[y][x] = BLACK;
		}
	}
}

void reset_board(board_type b)
{
	int x, y;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] != BLACK) {
				b[y][x] = WHITE;
			}
		}
	}
}

void print_board(board_type b)
{
	int x, y;
	
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			printf("%2d ", b[y][x]);
		}
		putchar('\n');
	}
}

void print_board2(board_type b)
{
	int x, y;
	
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == BLACK)
				printf("   ");
			else
				printf("%2u ", b[y][x]);
		}
		putchar('\n');
	}
}

int find_cell(board_type b, int *x, int *y, cell_type n)
{
	int xx, yy;
	for (yy = *y; yy < ROWS; yy ++) {
		for (xx = *x; xx < COLS; xx ++) {
			if (b[yy][xx] == n) {
				*x = xx;
				*y = yy;
				return 1;
			}
		}
	}
	return 0;
}

int is_vertical_begin(board_type b, int x, int y)
{
	return (y == 0 || b[y - 1][x] == BLACK) && y < ROWS - 1 &&
		b[y + 1][x] != BLACK;
}

int is_horizontal_begin(board_type b, int x, int y)
{
	return (x == 0 || b[y][x - 1] == BLACK) && x < COLS - 1 &&
		b[y][x + 1] != BLACK;
}

int check_box(board_type b, int x, int y, cell_type n)
{
	if (x > 0) {
		if (y > 0) {
			if (b[y - 1][x - 1] == n && b[y][x - 1] == n && b[y - 1][x] == n) {
				return 1;
			}
		}
		if (y < ROWS - 1) {
			if (b[y + 1][x - 1] == n && b[y][x - 1] == n && b[y + 1][x] == n) {
				return 1;
			}
		}
	}
	if (x < COLS - 1) {
		if (y > 0) {
			if (b[y - 1][x + 1] == n && b[y][x + 1] == n && b[y - 1][x] == n) {
				return 1;
			}
		}
		if (y < ROWS - 1) {
			if (b[y + 1][x + 1] == n && b[y][x + 1] == n && b[y + 1][x] == n) {
				return 1;
			}
		}
	}
	return 0;
}

int check_line_length(board_type b, int x, int y)
{
	int xx, yy, c = 1;
	for (xx = x - 1; xx >= 0 && b[y][xx] != BLACK; xx --, c ++) {
		if (c >= MAXLENGTH) return 1;
	}
	for (xx = x + 1; xx < COLS && b[y][xx] != BLACK; xx ++, c ++) {
		if (c >= MAXLENGTH) return 1;
	}
	c = 1;
	for (yy = y - 1; yy >= 0 && b[yy][x] != BLACK; yy --, c ++) {
		if (c >= MAXLENGTH) return 1;
	}
	for (yy = y + 1; yy < ROWS && b[yy][x] != BLACK; yy ++, c ++) {
		if (c >= MAXLENGTH) return 1;
	}
	return 0;
}

int count_line_length(board_type b, int x, int y, int dx, int dy)
{
	int c = 0;
	for (; x < COLS && y < ROWS && b[y][x] != BLACK; x += dx, y += dy) {
		c ++;
	}
	return c;
}

int check_black_box(board_type b)
{
	int x, y;
	for (y = 1; y < ROWS; y += 2) {
		for (x = 1; x < COLS; x += 2) {
			if (b[y][x] == BLACK && check_box(b, x, y, -1)) return 1;
		}
	}
	return 0;
}

void check_split_fill(board_type b, int x, int y)
{
	b[y][x] = FILLED;
	if (x > 0 && b[y][x - 1] == WHITE) {
		check_split_fill(b, x - 1, y);
	}
	if (x < COLS - 1 && b[y][x + 1] == WHITE) {
		check_split_fill(b, x + 1, y);
	}
	if (y > 0 && b[y - 1][x] == WHITE) {
		check_split_fill(b, x, y - 1);
	}
	if (y < ROWS - 1 && b[y + 1][x] == WHITE) {
		check_split_fill(b, x, y + 1);
	}
}

int check_split(board_type b)
{
	int x, y, c;
	x = 0; y = 0;
	if (! find_cell(b, &x, &y, WHITE)) return 1;
	check_split_fill(b, x, y);
	c = 0;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == FILLED) {
				b[y][x] = WHITE;
			}
			else if (b[y][x] == WHITE) {
				c ++;
			}
		}
	}
	return c;
}

int check_link_count(board_type b)
{
	int x, y, c, i;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] != WHITE) continue;
			if (is_vertical_begin(b, x, y)) {
				c = 0;
				for (i = y; i < ROWS && b[i][x] == WHITE; i ++) {
					if ((x > 0 && b[i][x - 1] == WHITE) ||
					    (x < COLS - 1 && b[i][x + 1] == WHITE)) {
						c ++;
					}
				}
				if (c < 2) return 1;
			}
			if (is_horizontal_begin(b, x, y)) {
				c = 0;
				for (i = x; i < COLS && b[y][i] == WHITE; i ++) {
					if ((y > 0 && b[y - 1][i] == WHITE) ||
					    (y < ROWS - 1 && b[y + 1][i] == WHITE)) {
						c ++;
					}
				}
				if (c < 2) return 1;
			}
		}
	}
	return 0;
}

int check_dig(board_type b, int x, int y)
{
	return ! check_box(b, x, y, WHITE) && ! check_line_length(b, x, y);
}

int dig(board_type b, int x, int y)
{
	int xx, yy;
	xx = COLS - 1 - x; yy = ROWS - 1 - y;
	if (b[y][x] != BLACK || b[yy][xx] != BLACK) return 0;
	b[y][x] = b[yy][xx] = WHITE;
	if (check_dig(b, x, y) && check_dig(b, xx, yy)) return 1;
	b[y][x] = b[yy][xx] = BLACK;
	return 0;
}

int select_dig(board_type b)
{
	unsigned short s[ROWS * COLS];
	int i, n, x, y;
	
	n = 0;
	for (y = ROWS / 2; y >= 0; y --) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == WHITE) {
				s[n ++] = (y << 8) | (x & 0xff);
			}
		}
	}
	if (ROWS % 2) {
		y = ROWS / 2 + 1;
		for (x = COLS / 2 + COLS % 2; x >= 0; x --) {
			if (b[y][x] == WHITE) {
				s[n ++] = (y << 8) | (x & 0xff);
			}
		}
	}
	if (n == 0) return 0;
	rand_shuffle(s, n);
	for (i = 0; i < n; i ++) {
		enum direction d = 0xf, l;
		x = s[i] & 0xff; y = s[i] >> 8;
		if (y == 0) d ^= UP;
		if (y == ROWS - 1) d ^= DOWN;
		if (x == 0) d ^= LEFT;
		if (x == COLS - 1) d ^= RIGHT;
		while (d) {
			l = rand_select_bit(d);
			switch (l) {
			case UP:
				if (dig(b, x, y - 1)) return 1;
				break;
			case DOWN:
				if (dig(b, x, y + 1)) return 1;
				break;
			case LEFT:
				if (dig(b, x - 1, y)) return 1;
				break;
			case RIGHT:
				if (dig(b, x + 1, y)) return 1;
				break;
			default:
				return 0;
			}
			d ^= l;
		}
	}
	return 0;
}

void init_board(board_type b)
{
	clear_board(b);
	dig(b, 0, 0);
}

void print_lines(struct line *lines, int nlines)
{
	int i;
	for (i = 0; i < nlines; i ++) {
		printf("%d:(%d,%d)(%d,%d)%d\n", i,
			lines[i].x, lines[i].y, lines[i].dx, lines[i].dy,
			lines[i].length);
	}
}

int count_lines(board_type b)
{
	int x, y, l;
	l = 0;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == BLACK) continue;
			if (is_vertical_begin(b, x, y)) l ++;
			if (is_horizontal_begin(b, x, y)) l ++;
		}
	}
	return l;
}

int line_cmp(const void *lhs, const void *rhs)
{
	const struct line *l = lhs, *r = rhs;
	short lp = l->x + l->y, rp = r->x + r->y;
	if (lp < rp) return -1;
	if (lp > rp) return 1;
	if (l->x < r->x) return -1;
	if (l->x > r->x) return 1;
	return 0;
}

int set_lines(board_type b, struct line *lines, int n)
{
	int x, y, i;
	i = 0;
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == BLACK) continue;
			if (is_vertical_begin(b, x, y)) {
				assert(i < n);
				lines[i].x = x;
				lines[i].y = y;
				lines[i].dx = 0;
				lines[i].dy = 1;
				lines[i].length = count_line_length(b, x, y, 0, 1);
				i ++;
			}
			if (is_horizontal_begin(b, x, y)) {
				assert(i < n);
				lines[i].x = x;
				lines[i].y = y;
				lines[i].dx = 1;
				lines[i].dy = 0;
				lines[i].length = count_line_length(b, x, y, 1, 0);
				i ++;
			}
		}
	}
#ifndef NDEBUG
	puts("lines");
	print_lines(lines, i);
#endif
	qsort(lines, i, sizeof(struct line), line_cmp);
#ifndef NDEBUG
	puts("sorted lines");
	print_lines(lines, i);
#endif
	return i;
}

void print_nums(struct number *nums, int nlines)
{
	int i, j, k = 0, l = 0;
	for (i = 0; i < nlines; i ++) {
		if (nums[i].length != l) {
			printf("\n- %d -\n", nums[i].length);
			k = 0;
		}
		else if (k > 59) {
			putchar('\n');
			k = 0;
		}
		l = nums[i].length;
		for (j = 0; j < nums[i].length; j ++, k ++) {
			printf("%d", nums[i].digit[j]);
		}
		putchar(' ');
		k ++;
	}
	if (k > 1) {
		putchar('\n');
	}
}

void set_number_index(struct line *lines, short *idx, int n)
{
	int i;
	short a, b;
	for (i = 0; i < MAXLENGTH - 1; i ++) idx[i] = 0;
	for (i = 0; i < n; i ++) idx[lines[i].length - 2] ++;
	a = 0;
	for (i = 0; i < MAXLENGTH - 1; i ++) {
		b = idx[i]; idx[i] = a; a += b;
	}
#ifndef NDEBUG
	puts("set_number_index");
	for (i = 0; i < MAXLENGTH - 1; i ++) {
		printf("%i:%d\n", i, idx[i]);
	}
#endif
}

int num_cmp(const void *lhs, const void *rhs)
{
	const struct number *l = lhs, *r = rhs;
	if (l->length < r->length) return -1;
	if (l->length > r->length) return 1;
	return memcmp(l->digit, r->digit, sizeof(short) * l->length);
}

int fill_number(board_type b, struct line *lines, struct number *nums,
	int nlines, int max, int (*fn)(void *), void *data)
{
	int x, y, i, j, k;
	cell_type o[MAXLENGTH];
	for (i = 0; i < nlines; i ++) {
		nums[i].length = lines[i].length;
		for (j = 0; ; j ++) {
			if (j > max) return 1;
			if ((*fn)(data)) return -1;
			x = lines[i].x; y = lines[i].y;
			for (k = 0; k < lines[i].length; k ++) {
				o[k] = b[y][x];
				if (b[y][x] == WHITE) {
					b[y][x] = rand_range(9) + 1;
				}
				nums[i].digit[k] = b[y][x];
				x += lines[i].dx; y += lines[i].dy;
			}
			for (k = 0; k < i; k ++) {
				if (! num_cmp(&nums[i], &nums[k])) break;
			}
			if (k == i) break;
			x = lines[i].x; y = lines[i].y;
			for (k = 0; k < lines[i].length; k ++) {
				b[y][x] = o[k];
				x += lines[i].dx; y += lines[i].dy;
			}
		}
	}
	qsort(nums, nlines, sizeof(struct number), num_cmp);
#ifndef NDEBUG
	print_board2(b);
	print_nums(nums, nlines);
#endif
	return 0;
}

int check_duplicate_num(struct number *nums, int nlines)
{
	int i;
	for (i = 1; i < nlines; i ++) {
		if (! num_cmp(&nums[i - 1], &nums[i])) return 1;
	}
	return 0;
}

int check_solution(board_type b, struct line *lines, struct number *nums,
	short *idx, unsigned short *result, int nlines)
{
	int i, j, x, y;
	cell_type o[MAXLENGTH];
	
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			if (b[y][x] == WHITE) return 0;
		}
	}
	for (i = 0; i < nlines; i ++) {
		result[i] = 0;
	}
	for (i = 0; i < nlines; i ++) {
		x = lines[i].x; y = lines[i].y;
		for (j = 0; j < lines[i].length; j ++) {
			o[j] = b[y][x];
			x += lines[i].dx; y += lines[i].dy;
		}
		for (j = idx[lines[i].length - 2]; ; j ++) {
			if (j >= nlines || nums[j].length > lines[i].length) {
				return 0;
			}
			if (result[j]) continue;
			if (! memcmp(o, nums[j].digit, sizeof(cell_type) * nums[j].length)) {
				result[j] = i + 1;
				break;
			}
		}
	}
#ifndef NDEBUG
	puts("completed");
	for (i = 0; i < nlines; i ++) {
		printf("%u\n", result[i]);
	}
#endif
	return 1;
}

int check_duplicate_solution(board_type b, struct line *lines, struct number *nums,
	short *idx, unsigned short *sol, int nlines, int (*fn)(void *), void *data)
{
	int i, j, x1, y1, x2, y2;
	cell_type o;
	
	for (i = 0, y1 = 0, x1 = 0; i < ROWS * COLS; i ++, x1 ++) {
		if (x1 == COLS) { y1 ++; x1 = 0; }
		for (j = i + 1, y2 = y1, x2 = x1 + 1; j < ROWS * COLS; j ++, x2 ++) {
			if ((*fn)(data)) return -1;
			if (x2 == COLS) { y2 ++; x2 = 0; }
			if (b[y1][x1] != b[y2][x2]) {
				o = b[y1][x1]; b[y1][x1] = b[y2][x2]; b[y2][x2] = o;
				if (check_solution(b, lines, nums, idx, sol, nlines)) {
					return 1;
				}
				o = b[y1][x1]; b[y1][x1] = b[y2][x2]; b[y2][x2] = o;
			}
		}
	}
	return 0;
}


int check_solve(int step, board_type b, struct line *lines,
	struct number *nums, short *idx, int nlines, int (*fn)(void *), void *data)
{
	int c, i, j, x, y, o[MAXLENGTH];
	c = 0;
	for (i = idx[lines->length - 2]; i < nlines && c < 2; i ++) {
		if ((*fn)(data)) return -1;
		if (nums[i].length == 0) continue;
		if (nums[i].length > lines->length) break;
		x = lines->x; y = lines->y;
		for (j = 0; j < lines->length; j ++) {
			if (b[y][x] != WHITE && b[y][x] != nums[i].digit[j]) break;
			x += lines->dx; y += lines->dy;
		}
		if (j < lines->length) continue;
		x = lines->x; y = lines->y;
		for (j = 0; j < lines->length; j ++) {
			o[j] = b[y][x];
			b[y][x] = nums[i].digit[j];
			x += lines->dx; y += lines->dy;
		}
		nums[i].length = 0;
		if (step == 1) {
#ifndef NDEBUG
			print_board2(b); putchar('\n');
#endif
			c ++;
		}
		else {
			c += check_solve(step - 1, b, lines + 1, nums, idx, nlines, fn, data);
		}
		x = lines->x; y = lines->y;
		for (j = 0; j < lines->length; j ++) {
			b[y][x] = o[j];
			x += lines->dx; y += lines->dy;
		}
		nums[i].length = lines->length;
	}
	return c;
}

void set_line_index(struct line *lines, int nlines, index_type idx)
{
	int i, j, k, x, y;
	
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			idx[y][x] = 0;
		}
	}
	for (i = 0; i < nlines; i ++) {
		x = lines[i].x; y = lines[i].y;
		k = (lines[i].dx == 0) ? 0 : 8;
		for (j = 0; j < lines[i].length; j ++) {
			idx[y][x] |= ((i + 1) << k);
			x += lines[i].dx; y += lines[i].dy;
		}
	}
#ifndef NDEBUG
	puts("line index");
	for (y = 0; y < ROWS; y ++) {
		for (x = 0; x < COLS; x ++) {
			printf("(%d,%d) ", idx[y][x] >> 8, idx[y][x] & 0xff);
		}
		putchar('\n');
	}
#endif
}

int get_vertical_line(index_type idx, int x, int y)
{
	return idx[y][x] & 0xff;
}

int get_horizontal_line(index_type idx, int x, int y)
{
	return idx[y][x] >> 8;
}


int can_set_number(board_type b, struct line *l, struct number *n)
{
	int i, x, y;
	if (l->length != n->length) return 0;
	x = l->x; y = l->y;
	for (i = 0; i < l->length; i ++) {
		if (b[y][x] != WHITE && b[y][x] != n->digit[i]) return 0;
		x += l->dx; y += l->dy;
	}
	return 1;
}


int set_number(board_type b, struct line *l, struct number *n)
{
	int x, y, i;
	if (can_set_number(b, l, n)) {
		x = l->x; y = l->y;
		for (i = 0; i < l->length; i ++) {
			b[y][x] = n->digit[i];
			x += l->dx; y += l->dy;
		}
		return 1;
	}
	return 0;
}


